rm(list=ls())

###############################################################
####
####  Copy Number recognition in Affymetrix SNP 6.0 arrays
####
#### 
####
##################################################################

### !!!!! Put the PATH to your data  !!!!

pPath<-"C:/GAP_AffymetrixSNP6_workflow_homepage/"

inputPath<-paste(pPath,"Vectors_and_Segmentations/",sep="")
outputPath<-paste(pPath,"Results_Plots/",sep="")
output_infoFileName<-"GAPrecognitionInfo_Affy6.txt"

### Annotation files
### !!!!! Correct the names of annotation files according to script_0  !!!!

CN_annot<-read.csv(paste(pPath,"CN_annot_Affy6_Version.csv",sep=""),header=T,fill=T,stringsAsFactors = F,row.names=1)
SNP_annot<-read.csv(paste(pPath,"SNP_annot_Affy6_Version.csv",sep=""),header=T,fill=T,stringsAsFactors = F,row.names=1)
CN_annot<-data.matrix(CN_annot)
SNP_annot<-data.matrix(SNP_annot)
C_Ind<-1;C_Chr<-2;C_Pos<-4


### Both source files need SNP_annot file defined above
source(paste(pPath,"script_functions_GAP_recognition_Affy6.R",sep=""))
source(paste(pPath,"script_functions_GAP_recognition_Affy6_fit.R",sep=""))


### !!!!! If you want, put some other file name for summary file  !!!!
output_infoFileName<-"GAPrecognitionInfo_Affy6.txt"

### !!!!! Check the GAP_parameters_file_name, te be the same as after script_3   !!!!
GAP_parameters_file_name<-"GAP_parameters_Example.csv"


if(file.access(paste(inputPath,GAP_parameters_file_name,sep=""), mode = 0)==0){
	GAP_par_exist<-T
	GAP_parameters_file<-read.csv(paste(inputPath,GAP_parameters_file_name,sep=""),stringsAsFactors = F,header=T,row.names=1)
	GAP_parameters_file<-data.matrix(GAP_parameters_file)
}else{GAP_par_exist<-F}


######## Structure of output summary information file concerning recognition
######## and rearrangements for all samples in the list
info<-c("p_BAF","q_LRR","2copyLRR","Breaks","DNAindex","Ploidy_detected","Chromosome_counts")
cat(c("SampleID",info), file = paste(outputPath,output_infoFileName,sep=""),sep="\t","\n")


#### !!!!! All segm_Sample1 files from Vectors_and_Segmentations folder will be processed !!!!

segFiles <- list.files(inputPath, pattern=vect_Segm_pattern,full.names=T) 

fileNames <- gsub(inputPath,"",segFiles) 
fileNames <- gsub(vect_Segm_pattern,"",fileNames) 
fileNames <- gsub(".txt","",fileNames) 

fileNames

nSample<-length(segFiles)

##########################   MAIN CYCLE  ######################################

ii<-1

for(ii in 1:nSample){

print(paste(as.character(ii),"out of",as.character(nSample),"processing",fileNames[ii],sep="  "))

tmp<-read.delim(segFiles[ii],header=T,sep="\t")

plot(tmp[,c_adL],tmp[,c_cn],cex=tmp[,c_len]/quantile(tmp[,c_len],prob=0.9),
		main="GAP",xlab="Allelic Difference",ylab="Log ratio")

abline(v=0)
abline(-1,1)

print(paste("   fitting GAP  ",fileNames[ii]))
notNAs<-intersect(which(!is.na(tmp[,c_cn]+tmp[,c_adL])),which(tmp[,c_chr]<23))

##### Choose parameters from GAP_parameters_file

GAP_parameters<-F

if(GAP_par_exist){
ii_name<-grep(fileNames[ii],rownames(GAP_parameters_file))
if(length(ii_name)!=1){
if(length(ii_name)>1){
	ii_name1<-numeric(0)
	for(i in 1:length(ii_name)){ii_name1<-c(ii_name1,grep(rownames(GAP_parameters_file)[ii_name[i]],fileNames[ii]))}
	if(length(ii_name1)==1){ii_name<-ii_name[ii_name1];GAP_parameters<-T}
}
}else{GAP_parameters<-T}
}


if(GAP_parameters){

### If precalculated GAP parameters: case 1.

p_BAF<-GAP_parameters_file[ii_name,2];q_LRR<-GAP_parameters_file[ii_name,3];Delta<-c(2,GAP_parameters_file[ii_name,4])
result<-CopyNumber_and_Genotype_patternWOfitting(tmp[notNAs,c_adL],tmp[notNAs,c_cn],tmp[notNAs,c_len],showGAPTemplate=T,power=3/4,p_BAF,q_LRR,Delta)

}else{

### If calculating GAP parameters de novo: case 3.

result<-CopyNumber_and_Genotype_pattern(tmp[notNAs,c_adL],tmp[notNAs,c_cn],tmp[notNAs,c_len],showGAPTemplate=T,power=3/4)

}
#### My parameters:
#### Delta is the vector of 2: c(copy number, LR value)
#### For example, sample BLC_T31 automatically detected as diploid, but, it looks like just dublicated diploid (near-tetraploid)
#### so, we introduce: p_BAF<-0.7;q_LRR<-0.16;Delta<-c(2,-0.1)
#### result<-CopyNumber_and_Genotype_patternWOfitting(tmp[notNAs,c_adL],tmp[notNAs,c_cn],tmp[notNAs,c_len],showGAPTemplate=T,power=3/4,p_BAF,q_LRR,Delta)

### coefficients of the pattern shrinkage
p_BAF<-result$coeff[1]
q_LRR<-result$coeff[2]

### matrix of coordinates of claster centers
Centr_CL<-result$Centr_CL

### matrix of clusters on the LRR x BAF plane
NETM<-result$NETM

### matrix of coordinates of subclones
SB_CL<-result$SB_CL


rm(result)

####### Final polish and output

fName<-paste(outputPath,"proc_",fileNames[ii],".txt",sep="")
tmp<-read.delim(fName,header=T,sep="\t")
tmp<-tmp[,-c(4,5)]

tmp[,c_ind]<-seq(1,dim(tmp)[[1]])
tmp<-getConfidence(tmp)



############# OUTPUT ##################################

###### Smoothing and saving the result

print(paste("   saving results  ",fileNames[ii]))

datA<-read.table(sub(vect_Segm_pattern,vect_AD_pattern,segFiles[ii]),header=F)
datA<-data.matrix(datA)
datA<-as.numeric(datA)
datA<-abs(datA)

datC<-read.table(sub(vect_Segm_pattern,vect_CN_pattern,segFiles[ii]),header=F)
datC<-data.matrix(datC)
datC<-as.numeric(datC)

CHR<-CHR_table;

tmp<-updateMeans_FinalPolish(tmp,datA,datC,Centr_CL,p_BAF)


####### Summary figures

tmp<-shrinkReprTMP(tmp)

GAPplot(tmp,nameSample=fileNames[ii],Centr_CL,NETM,q_LRR)

summaryPlots_perChrom(tmp,nameSample=fileNames[ii], datA, datC, Centr_CL)

info<-summaryPlot(tmp,nameSample=fileNames[ii], datA, datC, Centr_CL,NETM,q_LRR)

cat(c(fileNames[ii],info), file = paste(outputPath,output_infoFileName,sep=""),sep="\t",append=T,"\n")

}##ii

############  END MAIN CYCLE  #############################################

