###
###  FUNCTIONS, PACKAGES and GLOBAL VARIABLES for GAP recogntion
###

#########################################################################
##################
################## Packages
##################
#########################################################################

local({pkg <- "diptest"; if(nchar(pkg)) library(pkg, character.only=TRUE)})


#########################################################################
##################
################## Global variables
##################
#########################################################################

### Maximal copy number  
maxCN<-8

####### Columns of the main table with segmented profiles

#"Index" "IndxSnpStart" "IndxSnpEnd" "IndxCnStart" "IndxCnEnd" "Chr" "LenSnp" "CN_BP" "AD_BP" "CN" "AllDiff_L" "AllDiff_H"

####  "Index"		index of the segment
####  "IndxSnpStart"    segment starting SNP number in ordered by Chromosome and Position Affy 6.0 chip SNP annotation
####  "IndxSnpEnd"      segment ending SNP number in ordered by Chromosome and Position  Affy 6.0 chip SNP annotation
####  "IndxCnStart"   	segment starting CN number in ordered by Chromosome and Position  Affy 6.0 chip SNP annotation
####  "IndxCnEnd"	segment ending CN number in ordered by Chromosome and Position  Affy 6.0 chip SNP annotation
####  "Chr"             Chromosome number, integer for p arm, integer+0.5 for q arm,
####				X: 23, Y: 24, XY: 25
####  "LenSNP"		Length of a segment in SNP counts
####  "CN_BP"		Break point in CN profile
####  "AD_BP"		Break point in AD profile
####  "CN",             Median of LRR in the segment
####  "AllDiff_L",	One (any) of two AD modes in the segment, 
####  "AllDiff_sm"	Confidence of AD mode 
####  "nCN","nBA","nScn" Subsidiary columns, used for recognition,
####  			"nScn" contains smoothed CN signal according to FINAL PREDICTION
####  "Copy","Mallele"  FINAL PREDICTION OF COPY NUMBER and MAJOR ALLELE
####  "Segment_N", 	Segment number according to FINAL PREDICTION
####  "Subclone" 		subclone information (may be absent in the last version)
####  "CN1","BA1","Rank1" First choice prediction
####  "CN2","BA2","Rank2" Second choice prediction
####  "CN3","BA3","Rank3" Third choice prediction



# "Index","IndxSnpStart","IndxSnpEnd","IndxCnStart","IndxCnEnd",
c_ind<-1; c_is<-2; c_if<-3; c_isCN<-4; c_ifCN<-5

# "LenSnp","Chr",
c_chr<-6;c_len<-7; 

# "CN_BP" and "AD_BP"  
c_cnBP<-8; c_adBP<-9;

# "CN","AllDiff_L","AllDiff_sm",
c_cn<-10; c_adL<-11; c_adH<-12

# "nCN","nBA","nScn"
c_nC<-13; c_nA<-14; c_nS<-15

# "Copy","Mallele","Segment_N","Subclone",
c_CN<-16; c_BA<-17; c_conf<-18; c_sb<-19

# "CN1","BA1","Rank1","CN2","BA2","Rank2","CN3","BA3","Rank3" 
c_CN1<-20; c_BA1<-21; c_rnk1<-22


ColNamesTMP<-c("Index", "IndxSnpStart", "IndxSnpEnd", "IndxCnStart", "IndxCnEnd",
 "Chr", "LenSnp", "CN_BP", "AD_BP", "CN", "AllDiff_L", "AllDiff_sm",
"nCN","nAD","nScn","Copy","Mallele","Segment_N","Subclone",
"CN1","BA1","Rank1","CN2","BA2","Rank2","CN3","BA3","Rank3")


# pattern of filename with Log ratio vector 
vect_LR_pattern="V_LRR_"

# pattern of filename with Allelic Differences vector 
vect_AD_pattern="V_AD_"

# pattern of filename with Copy number (CN probes in Affy SNP6) vector 
vect_CN_pattern="V_CN_"

# pattern of filename for resulting Breakpoints table 
vect_BP_pattern="resBP_"

# pattern of filename for resulting segmented summary file 
vect_Segm_pattern="segm_"



#########################################################################
##################
################## Function get table of chromosome SNP counts in SNP_annot
##################
#########################################################################

CHR_table<-function(){

tt<-table(SNP_annot[,C_Chr],SNP_annot[,C_Chr+1])
CHR<-c(0);tCHR<-c(0)

for(k in 1:dim(tt)[[1]]){
	CHR<-c(CHR,CHR[2*k-1]+tt[k,1]);CHR<-c(CHR,CHR[2*k]+tt[k,2])
	tCHR<-c(tCHR,as.numeric(rownames(tt))[k])
	tCHR<-c(tCHR,as.numeric(rownames(tt))[k]+0.5)
}

tt<-which(tCHR==21)
names(CHR)<-tCHR;rm(tCHR);recCHR<-CHR;if(length(tt)>0){CHR<-CHR[-tt]}

out<-CHR

}


#############################################################
######
######  function CHR_table_recognition
######
##############################################################


CHR_table_recognition<-function(){

tt<-table(SNP_annot[,C_Chr],SNP_annot[,C_Chr+1])
CHR<-c(0);tCHR<-c(0)

for(k in 1:dim(tt)[[1]]){
	CHR<-c(CHR,CHR[2*k-1]+tt[k,1]);CHR<-c(CHR,CHR[2*k]+tt[k,2])
	tCHR<-c(tCHR,as.numeric(rownames(tt))[k])
	tCHR<-c(tCHR,as.numeric(rownames(tt))[k]+0.5)
}

tt<-which(tCHR==21)
names(CHR)<-tCHR;rm(tCHR);recCHR<-CHR;if(length(tt)>0){CHR<-CHR[-tt]}

out<-recCHR

}



#############################################################
######
######  function CHR_table_recognition
######
##############################################################


CHR_table_recognition_modif<-function(tmp){

tt2<-as.numeric(rownames(table(SNP_annot[,C_Chr]+(SNP_annot[,C_Chr+1]-1)/2)))
tt1<-as.numeric(rownames(table(tmp[,c_chr])))


if(length(tt1)!=length(tt2)){
	ttc<-setdiff(tt2,tt1)
	for(j in 1:length(ttc)){
		ttn<-which(SNP_annot[,C_Chr]==ttc[j])
		if(j==1){ttk<-ttn}else{ttk<-c(ttk,ttn)}
	}
}else{ttk<-numeric(0)}


if(length(ttk)>0){tt<-table(SNP_annot[-ttk,C_Chr],SNP_annot[-ttk,C_Chr+1])}else{tt<-table(SNP_annot[,C_Chr],SNP_annot[,C_Chr+1])}


#tt<-table(SNP_annot[,C_Chr],SNP_annot[,C_Chr+1])
CHR<-c(0);tCHR<-c(0)

for(k in 1:dim(tt)[[1]]){
	CHR<-c(CHR,CHR[2*k-1]+tt[k,1]);CHR<-c(CHR,CHR[2*k]+tt[k,2])
	tCHR<-c(tCHR,as.numeric(rownames(tt))[k])
	tCHR<-c(tCHR,as.numeric(rownames(tt))[k]+0.5)
}

tt<-which(tCHR==21)
names(CHR)<-tCHR;rm(tCHR);recCHR<-CHR;if(length(tt)>0){CHR<-CHR[-tt]}

out<-recCHR

}



#########################################################################
##################
################## Function smoothNET
##################
#########################################################################

smoothNET<-function(NET,weightC){

nr<-dim(NET)[[1]]
nf<-dim(NET)[[2]]

for(k in 1:2){
	NET1<-NET
	for(j in 2:(nf-1)){
		for(p in 2:(nr-1)){
			NET1[p,j]<-(sum(NET[(p-1):(p+1),(j-1):(j+1)])+weightC*NET[p,j])/(9+weightC)
			if((NET[p,j]>50)&&(NET1[p,j]<50)){NET1[p,j]<-NET[p,j]}
		}#p
		NET1[1,j]<-sum(NET[1:2,(j-1):(j+1)])/6
		NET1[nr,j]<-sum(NET[(nr-1):nr,(j-1):(j+1)])/6
	}#j
	for(p in 2:(nr-1)){
		NET1[p,1]<-sum(NET[(p-1):(p+1),1:2])/6
		NET1[p,nf]<-sum(NET[(p-1):(p+1),(nf-1):nf])/6
	}#p

	NET1[1,1]<-(NET[1,1]+NET[2,1]+NET[1,2]+NET[2,2])/4
	NET1[nr,nf]<-(NET[nr,nf]+NET[nr-1,nf]+NET[nr,nf-1]+NET[nr-1,nf-1])/4

	NET<-NET1
}#k

rm(NET1)
out<-NET
out
}



#########################################################################
##################
################## Function Centr_CL_annot
##################
#########################################################################


################################################
### 
###  Initializing Centr_CL, matrix of GAP cluster positions:
###  0/0  1/1  2/1 2/2   3/2 3/3  4/2 4/3 4/4   5/3 5/4 5/5   6/3 6/4 6/5 6/6   7/4 7/5 7/6 7/7   8/4 8/5 8/6 8/7 8/8
###  1 row is the number of copies, 2 row is the number of major alleles,  3 row is index 1:25
###  4,5... BAF and LRR coordinates of corresponding cluster 
###  
##########################################################################


Centr_CL_annot <- function(CN){

out<-matrix(0,2,2)
k<-2
for(cn in 2:CN){
out<-cbind(out,matrix(0,2,k))
if(cn%%2==1){
k<-k+1
}
}

for(cn in 1:(CN+1)){
	if(cn-1<=2){cn_CL<-cn-1}

	if(cn-1>2){
		cn_CL<-2
		for(k in 2:(cn-1)){
			for(nbc in trunc(k/2+1):k){cn_CL<-cn_CL+1}
		}
		cn_CL<-cn_CL-1
	}

	for(nbc in 1:trunc((cn-1)/2+1)){
	out[1,cn_CL+nbc]<-cn-1
	out[2,cn_CL+nbc]<-trunc((cn-1+1)/2)+nbc-1
	}
}# cn
out<-rbind(out,seq(1,dim(out)[[2]]))
out<-rbind(out,matrix(0,2,dim(out)[[2]]))

rownames(out)<-c("Copy","Major allele","Index","BAF","LRR")
out
}


#########################################################################
##################
################## Functions plotGAPdensity and plotGAPrecognition
##################
#########################################################################


plotGAPdensity<-function(NET){

NR<-dim(NET)[[1]]
NF<-dim(NET)[[2]]
SR<-as.numeric(rownames(NET))
SF<-as.numeric(colnames(NET))

plot(c(SF[1],SF[NF]),c(min(-1,SR[1]),SR[NR]),type="n",main="GAP",xlab="Allelic difference (AD)",ylab="Log ratio (CN)")


NETM<-as.numeric(NET)
NETM<-cbind(NETM,NETM,NETM)
NETM[,1]<-rep(seq(1,length(SR)),length(SF))
NETM[,2]<-rep(seq(1,length(SF)),each=length(SR))
NETM<-NETM[-which(NETM[,3]<10),]
NETM[,3]<-log2(NETM[,3])
NETM<-cbind(NETM,NETM[,3])
NETM<-NETM[order(NETM[,3],decreasing=F),]


hh<-hist(NETM[,3],breaks=64,plot=F)$breaks

p<-1
k<-1
while(p<=length(hh)){
while((k<dim(NETM)[[1]])&&(NETM[k,3]<hh[p])){
NETM[k,4]<-p
k<-k+1
}
p<-p+1
}

tt<-which(NETM[1,]==NR)
if(length(tt)>0){NETM[1,tt]<-NR-1}

tt<-which(NETM[2,]==NF)
if(length(tt)>0){NETM[2,tt]<-NF-1}

RB<-rainbow(length(hh))
RB[1]<-"#FF000000"

for(p in 1:dim(NETM)[[1]]){
rect(SF[NETM[p,2]],SR[NETM[p,1]],SF[NETM[p,2]+1],SR[NETM[p,1]+1], col=RB[NETM[p,4]],border=RB[NETM[p,4]])
}

}



plotGAPrecognition<-function(seg_BAF,seg_LRR,seg_len,NETM,Centr_CL){

CEX<-seg_len/2000
tt<-which(CEX>4)
if(length(tt)>0){CEX[tt]<-4}

tt<-which(CEX<0.03)
if(length(tt)>0){CEX[tt]<-0}

plot(tmp[,c_adL],tmp[,c_cn],main="BLC_B1_T52",xlim=c(-1.5,1.5),ylim=c(-1,1),type="n",xlab="Allelic difference (AD)",ylab="Log ratio (CN)")
points(-seg_BAF,seg_LRR,cex=CEX,col="black")
points(seg_BAF,seg_LRR,cex=CEX,col="grey")

abline(h=0)
abline(v=0)
abline(-1,1)
abline(-1+p_BAF,1)


RB<-rainbow(40)

for(p in 1:16){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])}
}

for(p in 31:36){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])}
}



for(j in 1:dim(Centr_CL)[[2]]){

	if(Centr_CL[1,j]<=7){
	if(Centr_CL[1,j]==Centr_CL[2,j]*2){
		text(0.05,Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
	}else{text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))	}
}}


text(0.9,-0.8,paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=0.8)
text(0.9,-0.9,paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=0.8)

}


########################################################################
#################
################# Function SR_perfect
#################
#########################################################################

####################################################
###
### "power" determines initial pattern of LRR
###
### if(power==0){SR_perf<-c(-1,0,0.85,1.55,2.1,2.6,3.05,3.45) ### just an attempt :)
### if(power==1){SR_perf<-c(-1,0,1,2,3,4,5,6)
### if power=1/2, 3/4 etc, step for copy number is proportional to corresponding power of copy number
### if power=3/4:  SR_perf<-c(-1,0,0.68,1.28, 1.83,2.34,2.83,3.3)
### if power=1/2:  SR_perf<-c(-1,0,0.41,0.73,1,1.23,1.45,1.64)
####################################################


SR_perfectSQRT <- function (CN,power){

if(power==0){SR_perf<-c(-1,0,0.85,1.55,2.1,2.6,3.05,3.45)
}else{
	if(power==1){SR_perf<-c(-1,0,1,2,3,4,5,6)
	}else{
		SR_perf<-(0)
		for(cn in 2:CN){SR_perf<-c(SR_perf,(cn-1)^(power))}
		SR_perf<-(SR_perf-SR_perf[2])
		}
}
out<-SR_perf
out
}


SR_perfectLOG2 <- function (CN){

SR_perf<-(-1)
for(cn in 2:CN){SR_perf<-c(SR_perf,SR_perf[cn-1]-log2((cn-1)/cn))}
SR_perf<-SR_perf/2

out<-SR_perf
out
}


#########################################################################
##################
################## Function fittingGAP
##################
#########################################################################

#power<-1/2
fittingGAP <- function (NET,power){

NF<-dim(NET)[[2]]
NR<-dim(NET)[[1]]
SF<-as.numeric(colnames(NET))
SR<-as.numeric(rownames(NET))
step_SF<-SF[2]-SF[1]
step_SR<-SR[2]-SR[1]

SR_perf<-SR_perfectSQRT(CN=5,power)

qq_L<-c(seq(0.1,0.18,0.02),seq(0.2,0.45,0.05),seq(0.5,0.9,0.1),seq(1,max(1,round(max(SF)/max(SR_perf)-0.05,1)),0.2))

step_L<-0.02

pp_N<-seq(0.04,0.6,0.04)


lim_CL<-3

#### Initializing ouput matrix FF_all, with cluster position and quality criterium

FF_all<-matrix(0,24,2)

rownames(FF_all)<-c("Coverage","p_BAF","p_LRR","1","2","3","4","5",
"copy_2","clust34","homo_clust","5_3","4_0","4_4","4_3","4_2","3_0","3_3","3_2","2_0","2_2","2_1","1_0","1_1")

#### Cycle for fitting tumor DNA proportion

NET_tmp<-matrix(1,16,7)
colnames(NET_tmp)<-c("N_LRR","N_BAF","Step_LRR","Step_BAF_le","Step_BAF_ri","Copy","majorA")
NET_tmp[,6]<-c(5,5,5,5,4,4,4,4,3,3,3,2,2,2,1,1)
NET_tmp[,7]<-c(0,5,4,3,0,4,3,2,0,3,2,0,2,1,0,1)



# cycle for LRR contraction
kk<-16
for(kk in 1:length(qq_L)){

# cycle for contamination
jj<-1
	for(jj in 1:length(pp_N)){

		SR_C<-round(SR_perf*qq_L[kk],3)
		SR_C<-SR_C-(SR_C[1]+0.6)-step_L

# cycle for pattern shift

		while(SR_C[1]+step_L<0){
			SR_C<-SR_C+step_L


#### Determination GAP structure, such as cluster size in BAf 
#### and LRR scale for given coefficients

			NET_tmp[,3:5]<-NET_tmp[,3:5]*0

			NET_tmp[,1]<-SR_C[NET_tmp[,6]]
			NET_tmp[,2]<-SR_C[NET_tmp[,6]]+1
			
			tt<-c(8,14);NET_tmp[tt,2]<-0
			tt<-c(2,6,10,13,16);NET_tmp[tt,2]<-NET_tmp[tt,2]-pp_N[jj]

			if(NET_tmp[16,2]>0.1){

			tt<-tt[1:3];NET_tmp[tt+1,2]<-NET_tmp[tt,2]/NET_tmp[tt,6]*(NET_tmp[tt+1,7]-1)
			tt<-tt[1];NET_tmp[tt+2,2]<-NET_tmp[tt,2]/NET_tmp[tt,6]*(NET_tmp[tt+2,7]-2)
		
			### coordinates of clusters
			NET_tmp[,1]<-round((NET_tmp[,1]-SR[1])/step_SR+1,0)
			NET_tmp[,2]<-round((NET_tmp[,2]-SF[1])/step_SF+1,0)

			### cluster size on the template
			### 1. LRR scale
			tt<-c(15,15,12,9,5);tt<-NET_tmp[tt,1];tt[1]<-0
			NET_tmp[,3]<-tt[NET_tmp[,6]]	
			NET_tmp[,3]<-NET_tmp[,1]-NET_tmp[,3]-1	
		
			NET_tmp[,3]<-round(NET_tmp[,3]/lim_CL,0)
			tt<-which(NET_tmp[,3]>15);if(length(tt)>0){NET_tmp[tt,3]<-15}
			tt<-which(NET_tmp[,3]<6);if(length(tt)>0){NET_tmp[tt,3]<-round(NET_tmp[tt,3]*3/2-0.6)}

			### 2. AD scale

			NET_tmp[1:(dim(NET_tmp)[[1]]-1),4]<-NET_tmp[1:(dim(NET_tmp)[[1]]-1),2]-NET_tmp[2:dim(NET_tmp)[[1]],2]-1
			tt<-which(NET_tmp[,4]<=0); if(length(tt)>0) {NET_tmp[tt,4]<-NET_tmp[tt,2]-1}
	
			NET_tmp[2:dim(NET_tmp)[[1]],5]<-NET_tmp[1:(dim(NET_tmp)[[1]]-1),2]-NET_tmp[2:dim(NET_tmp)[[1]],2]-1
			tt<-which(NET_tmp[,5]<=0); if(length(tt)>0) {NET_tmp[tt,5]<-NF-NET_tmp[tt,2]}
			tt<-which(NET_tmp[,5]<=0); if(length(tt)>0) {NET_tmp[tt,5]<-10}

			NET_tmp[,4:5]<-round(NET_tmp[,4:5]/2-0.1,0)
	
			tt<-which(NET_tmp[,4]>6);if(length(tt)>0){NET_tmp[tt,4]<-6}			
			tt<-which(NET_tmp[,5]>6);if(length(tt)>0){NET_tmp[tt,5]<-6}			

			### 2.1 AD scale: treat a little contamination as pure

			tt<-which(NET_tmp[,7]==0);
			
			tt1<-min(NET_tmp[tt,2]-NET_tmp[tt+1,2]-1)
			if(tt1<=(6+6+6)){
				tt1<-trunc(tt1/3)
				if(tt1>=3){
					NET_tmp[tt+1,5]<-tt1
					NET_tmp[tt,4]<-tt1
				}else{
					NET_tmp[tt+1,5]<-max(0,tt1)
					NET_tmp[tt+1,4]<-max(0,12-(NET_tmp[tt+1,5]+NET_tmp[tt,5]+1))
					NET_tmp[tt,4]<-max(0,(NET_tmp[tt,2]-NET_tmp[tt+1,2]-NET_tmp[tt+1,5]-1))
				}
				
				#NET_tmp[tt+1,5]<-max(0,tt1)
				#NET_tmp[tt+1,4]<-max(0,12-(NET_tmp[tt+1,5]+NET_tmp[tt,5]+1))
				#NET_tmp[tt,4]<-max(0,(NET_tmp[tt,2]-NET_tmp[tt+1,2]-NET_tmp[tt+1,5]-1))


			}
		
			NET_tmp[,4]<-NET_tmp[,2]-NET_tmp[,4]
			NET_tmp[,5]<-NET_tmp[,2]+NET_tmp[,5]

			tt<-which(NET_tmp[,4]<1);if(length(tt)>0){NET_tmp[tt,4]<-1}
			tt<-which(NET_tmp[,5]>NF)
			if(length(tt)>0){
				tt1<-which(NET_tmp[tt,1]<=NR)
				if(length(tt1)>0){NET_tmp[tt[tt1],5]<-NF}
			}

#for(p in 1:dim(NET_tmp)[[1]]){
#rect(SF[NET_tmp[p,4]],SR[NET_tmp[p,1]-NET_tmp[p,3]],SF[NET_tmp[p,5]],SR[NET_tmp[p,1]+NET_tmp[p,3]], col="grey",border="grey")
#}
#abline(-1,1)

#### Cycle for fitting GAP pattern position on NET

			pos<-length(which(NET_tmp[,1]>0))
			out<-length(which(NET_tmp[,1]>NR))
		
			if(pos>0){
			FF_max2<-rep(0,24)
			FF_max4<-rep(0,24)

			FF<-rep(0,24)

			if(out<pos){

			for(j in (out+1):pos){
				FF[8+j]<-sum(NET[max(1,(NET_tmp[j,1]-NET_tmp[j,3])):min(NR,(NET_tmp[j,1]+NET_tmp[j,3])),NET_tmp[j,4]:NET_tmp[j,5]])
			}#j
			FF[1]<-sum(FF)
			FF_max4<-c(FF[1],pp_N[jj],qq_L[kk],NET_tmp[c(15,12,9,5,1),1],FF[9:24])
		
			FF[1]<-sum(FF[17:24])
			FF_max2<-c(-FF[1],pp_N[jj],qq_L[kk],NET_tmp[c(15,12,9,5,1),1],FF[9:24])
		
			if((pos<dim(NET_tmp)[[1]])&&(NET_tmp[pos+1,1]>0)){
			pos<-length(which(NET_tmp[,1]>0))}

			if((out<dim(NET_tmp)[[1]])&&(NET_tmp[out+1,1]>NR)){
			out<-length(which(NET_tmp[,1]>NR))}

			if(FF_max2[1]<0){FF_all<-cbind(FF_all,FF_max2)}
			if(FF_max4[1]>0){FF_all<-cbind(FF_all,FF_max4)}

			}# if cluster in NET range
			}# if there are positive indexes
			}# if still afordable contamination
		}#SR_C
	}#jj
	
}#kk

#######################################################
#### Filtering resulting patterns

#### 1. Set clusters that are beyond the NET to 0
for(j in 4:8){
	if(length(which(FF_all[j,]<0))>0){FF_all[j,which(FF_all[j,]<0)]<-0}
	if(length(which(FF_all[j,]>NR))>0){FF_all[j,which(FF_all[j,]>NR)]<-0}
}

FF_all[9,]<-FF_all[20,]+FF_all[21,]+FF_all[22,]
FF_all[10,]<-FF_all[15,]+FF_all[19,]
FF_all[11,]<-FF_all[14,]+FF_all[18,]+FF_all[21,]+FF_all[24,]
FF_all[12,]<-sign(FF_all[14,]*FF_all[15,]*FF_all[18,]*FF_all[19,]*FF_all[2,])*500

#### 2. Removing patterns that either has no 2 copy level or no 3/2 and 4/3

#tt<-which(FF_all[9,]+FF_all[10,]<500); if(length(tt)>0){FF_all<-FF_all[,-tt]}

tt<-which(FF_all[9,]+FF_all[12,]<500); if(length(tt)>0){FF_all<-FF_all[,-tt]}

#### 3. Removing patterns that has nothing at tumor homozygous line

tt<-which(FF_all[10,]>500);if(length(tt)>0){FF_all[10,tt]<-1000}

tt<-which(FF_all[11,]-FF_all[10,]<0); if(length(tt)>0){FF_all<-FF_all[,-tt]}

#### 4. Order patterns by COVERAGE criterium and leave the first 200 solution
#### for further analysis

FF_all<-FF_all[1:11,]

tt<-which(FF_all[1,]<0)
if(length(tt)>0){FF_all2<-FF_all[,tt]
if(dim(FF_all)[[2]]>length(tt)){FF_all<-FF_all[,-tt]}else{
FF_all<-FF_all2
}

}else{FF_all2<-NA}


FF_all<-abs(FF_all)

#FF_all<-abs(FF_all2)

FF_all<-FF_all[,order(FF_all[1,],decreasing=T)]


######################## Summarizing FF_all

FF_all[9,]<-0
FF_all[10,]<-round(FF_all[1,]/FF_all[1,1]*100,1)
tt<-which(FF_all[10,]<70)
if(length(tt)>0){FF_all<-FF_all[,-tt]}


for(k in 1:(dim(FF_all)[[2]]-1)){
	if(FF_all[9,k]==0){FF_all[9,k]<-k;kn<-1
		
		for(kk in (k+1):dim(FF_all)[[2]]){
			if((max(abs(FF_all[4:8,k]-FF_all[4:8,kk]))<10)&&
				(abs(FF_all[2,k]-FF_all[2,kk])<0.1)){
				if((FF_all[10,k]-FF_all[10,kk]<5)&&(kn<10)){FF_all[9,kk]<-k
                        kn<-kn+1
                        }else{FF_all[9,kk]<-(-1)}
			}else{}
	}}
}

k<-dim(FF_all)[[2]]
if(FF_all[9,k]==0){FF_all[9,k]<-k}


tt<-which(FF_all[9,]==(-1))
if(length(tt)>1){
if(length(tt)>0){FF_all<-FF_all[,-tt]}
}else{FF_all<-cbind(FF_all,FF_all)
tt<-which(FF_all[9,]==(-1))
if(length(tt)>0){FF_all<-FF_all[,-tt]}
}


FF_all<-FF_all[,order(FF_all[9,],decreasing=F)]

#table(FF_all[9,])

FF_all<-FF_all[1:10,]
rownames(FF_all)<-c("Coverage_SNP","p_BAF","q_LRR","1_copy","2_copy","3_copy","4_copy","5_copy","Set_pattern","Percent_coverage")

out<-FF_all


#for(p in 4:8){
#abline(h=SR[FF_all[p,1]],col="red")
#}



}# function fit GAP


#########################################################################
##################
################## Function getCenterClusterGAP
##################
#########################################################################

#CN<-8
#power=3/4
#pp=0
#qq=0.42
#Delta=c(1,-0.27)

getCenterClusterGAP<-function(CN, pp, qq, Delta,power){

	Centr_CL<-Centr_CL_annot(CN)
	
	SR_perf<-SR_perfectSQRT(CN,power)
	SR_perf<-SR_perf*qq
	DD<-SR_perf[Delta[1]]-Delta[2]
	SR_perf<-c(-1,round((SR_perf-DD),3))

	for(cn in 1:9){Centr_CL[5,which(Centr_CL[1,]==(cn-1))]<-SR_perf[cn]}

	Centr_CL[4,1]<-0
	cn<-3
	for(cn in 1:8){
		max_F<-rep(0,cn+1)
		
		cn_CL<-1
		while(Centr_CL[1,cn_CL]<cn){cn_CL<-cn_CL+1}		
		max_F[cn+1]<-Centr_CL[5,cn_CL]-pp+1
#cn<-5
		if(cn>2){
			max_F[cn+1]<-max_F[cn+1]
			max_F[(trunc(cn/2+1)+1):cn]<-(1+(cn+1)%%2)-2

			for(nbc in trunc(cn/2+1):(cn-1)){
				max_F[(nbc+1):cn]<-max_F[(nbc+1):cn]+2
			}#nbc
		if(cn>4){
		max_F[(trunc(cn/2+1)+1):cn]<-((10-cn)*10-1)/((10-cn)*10)*max_F[cn+1]/cn*max_F[(trunc(cn/2+1)+1):cn]+max_F[cn+1]/((10-cn)*10)
		}else{
		max_F[(trunc(cn/2+1)+1):cn]<-max_F[cn+1]/cn*max_F[(trunc(cn/2+1)+1):cn]
			}
		}

		for(nbc in 1:trunc(cn/2+1)){
			Centr_CL[4,cn_CL+nbc-1]<-max_F[trunc((cn+1)/2)+nbc]
		}#nbc
	}#cn


#Centr_CL[4,2]<-1/2*((Centr_CL[5,2]+1-pp)+sqrt((Centr_CL[5,2]+1-pp)^2+4/50))
#*(Centr_CL[4,4]/(Centr_CL[5,4]+1))

Centr_CL[5,1]<-Centr_CL[5,1]+pp-0.1
if((Centr_CL[5,4]-Centr_CL[5,2])>(Centr_CL[5,2]-Centr_CL[5,1])){
Centr_CL[5,1]<-Centr_CL[5,1]-pp
}
Centr_CL[4:5,]<-round(Centr_CL[4:5,],3)

out<-Centr_CL

}# function getCenterClusterGAP



#########################################################################
##################
################## Function GAPonNET_template
##################
#########################################################################

GAPonNET_template<-function(Centr_CL,SR,SF){

NR<-length(SR)
NF<-length(SF)
step_SF<-SF[2]-SF[1]
step_SR<-SR[2]-SR[1]



NET_tmp<-matrix(0,33,9)
colnames(NET_tmp)<-c("N_LRR","N_BAF","Step_LRR_down","Step_LRR_up","Step_BAF_le","Step_BAF_ri","Copy","majorA","Clust_Num")

NET_tmp[1:dim(Centr_CL)[[2]],7:9]<-t(Centr_CL[1:3,])
NET_tmp[1:dim(Centr_CL)[[2]],1]<-Centr_CL[5,]
NET_tmp[1:dim(Centr_CL)[[2]],2]<-Centr_CL[4,]



p<-dim(Centr_CL)[[2]]
for(k in 1:max(Centr_CL[1,])){
NET_tmp[p+k,7]<-k
NET_tmp[p+k,1]<-Centr_CL[5,which(Centr_CL[1,]==k)[1]]
NET_tmp[p+k,2]<-Centr_CL[5,which(Centr_CL[1,]==k)[1]]+1
NET_tmp[p+k,9]<-30+k
}

NET_tmp[,1]<-round((NET_tmp[,1]-SR[1])/step_SR+1,0)
NET_tmp[,2]<-round((NET_tmp[,2]-SF[1])/step_SF+1,0)

NET_tmp<-NET_tmp[order(NET_tmp[,1],NET_tmp[,2],decreasing=T),]

### 1 LRR scale

tt<-as.numeric(rownames(table(NET_tmp[,1])))
tt<-c(min(tt)-2,tt,max(tt)+trunc(0.1/step_SR))

for(k in 2:(length(tt)-1)){
NET_tmp[which(NET_tmp[,1]==tt[k]),3]<-tt[k-1]
NET_tmp[which(NET_tmp[,1]==tt[k]),4]<-tt[k+1]
}

NET_tmp[,3]<-NET_tmp[,1]-NET_tmp[,3]
NET_tmp[,4]<-NET_tmp[,4]-NET_tmp[,1]

NET_tmp[,3:4]<-round((NET_tmp[,3:4]-2)/2.5,0)

tt<-which(NET_tmp[,3]>15);if(length(tt)>0){NET_tmp[tt,3]<-15}
tt<-which(NET_tmp[,3]<6);if(length(tt)>0){NET_tmp[tt,3]<-max(0,round(NET_tmp[tt,3]*2.5/2-0.6))}

tt<-which(NET_tmp[,4]>15);if(length(tt)>0){NET_tmp[tt,4]<-15}
tt<-which(NET_tmp[,4]<6);if(length(tt)>0){NET_tmp[tt,4]<-max(0,round(NET_tmp[tt,4]*2.5/2-0.6))}

### 2 AD scale

NET_tmp[1:(dim(NET_tmp)[[1]]-1),5]<-NET_tmp[1:(dim(NET_tmp)[[1]]-1),2]-NET_tmp[2:dim(NET_tmp)[[1]],2]-1
tt<-which(NET_tmp[,5]<=0); if(length(tt)>0) {NET_tmp[tt,5]<-NET_tmp[tt,2]-1}
	
NET_tmp[2:dim(NET_tmp)[[1]],6]<-NET_tmp[1:(dim(NET_tmp)[[1]]-1),2]-NET_tmp[2:dim(NET_tmp)[[1]],2]-1
tt<-which(NET_tmp[,6]<=0); if(length(tt)>0) {NET_tmp[tt,6]<-max(NET_tmp[,2]+trunc(0.1/step_SF))-NET_tmp[tt,2]}
	

NET_tmp[c(11,20,27,32),5]<-NET_tmp[c(11,20,27,32),5]*2
NET_tmp[,5:6]<-round(NET_tmp[,5:6]/4,0)
	
tt<-which(NET_tmp[1:31,5]>8);if(length(tt)>0){NET_tmp[tt,5]<-8}			
tt<-which(NET_tmp[1:31,6]>8);if(length(tt)>0){NET_tmp[tt,6]<-8}		

NET_tmp[32,5]<-min(10,NET_tmp[32,5])
NET_tmp[33,6]<-min(12,NET_tmp[33,6])


### 2.1 AD scale: enlarge balanced cluster window

tt<-which(NET_tmp[,5]==0);
if(length(tt)>0){
NET_tmp[tt,6]<-min(16,max(8,NET_tmp[11,2]-NET_tmp[11,5]-3))
}		

### 2.1 AD scale: treat a little contamination as pure

tt<-which(NET_tmp[1:32,8]==0);
tt1<-min(NET_tmp[tt,2]-NET_tmp[tt+1,2]-1)

if(tt1<=(8+6+8)){
	tt1<-trunc(tt1/3)
	if(tt1>=3){
		NET_tmp[tt+1,6]<-tt1
		NET_tmp[tt,5]<-tt1
	}else{
		NET_tmp[tt,2]<-NET_tmp[tt,2]+NET_tmp[tt,6]
		NET_tmp[tt,5]<-0
		NET_tmp[tt,6]<-1
		NET_tmp[tt+1,6]<-NET_tmp[tt,2]-NET_tmp[tt+1,2]-1
		NET_tmp[tt+1,5]<-17-NET_tmp[tt+1,6]
	}

}

tt<-which(NET_tmp[,5]<0);if(length(tt)>0){NET_tmp[tt,5]<-0}
tt<-which(NET_tmp[,6]<0);if(length(tt)>0){NET_tmp[tt,6]<-0}

out<-NET_tmp

}# function GAPonNET_template


#########################################################################
##################
################## Function markNETtemplate
##################
#########################################################################


markNETtemplate<-function(NET,NET_tmp){


NF<-dim(NET)[[2]]
NR<-dim(NET)[[1]]
SF<-as.numeric(colnames(NET))
SR<-as.numeric(rownames(NET))
step_SF<-SF[2]-SF[1]
step_SR<-SR[2]-SR[1]


NET1<-NET

NET1<-NET1*0
NET2<-NET1*0
NET3<-NET1*0

p<-1
for(p in 1:dim(NET_tmp)[[1]]){
	m<-(NET_tmp[p,1]-NET_tmp[p,3])
	k<-(NET_tmp[p,1]+NET_tmp[p,4])
	s<-(NET_tmp[p,2]-NET_tmp[p,5])
	n<-(NET_tmp[p,2]+NET_tmp[p,6])
	NET1[m:k,s:n]<-(-NET_tmp[p,9])
	NET3[m:k,s:n]<-(-NET_tmp[p,9])
	j<-50	
	while(j>=1){
		NET2[max(1,m-j):min(NR,k+j),max(1,s-j):min(NF,n+j)]<-j
		j<-j-1
	}	
	for(j in max(1,m-50):min(NR,k+50)){
		for(jj in max(1,s-50):min(NF,n+50)){
			if((NET1[j,jj]>0)&&(NET2[j,jj]<NET1[j,jj])||(NET1[j,jj]==0)){
				NET1[j,jj]<-NET2[j,jj]
				NET3[j,jj]<-NET_tmp[p,9]
			}
	}	}
}#p


for(j in 3:(dim(NET3)[[1]]-3)){
	for(p in 3:(dim(NET3)[[2]]-3-8)){
		tt<-as.numeric(abs(NET3[j:(j+1),p:(p+1)]))
		if(length(which(tt==0))>0){tt<-tt[-which(tt==0)]}
		if((length(tt)>0)&&(min(tt)!=max(tt))){
		NET3[(j-2):(j+3),(p-2):(p+3)]<-0
		}
}	}

for(j in 3:(dim(NET3)[[1]]-3)){
	for(p in (dim(NET3)[[2]]-3-8):(dim(NET3)[[2]]-1)){
		tt<-as.numeric(abs(NET3[j:(j+1),p:(p+1)]))
		if(length(which(tt==0))>0){tt<-tt[-which(tt==0)]}
		if((length(tt)>0)&&(min(tt)!=max(tt))){
			NET3[(j-2):(j+3),p:(p+1)]<-0
		}
}	}


for(j in 1:dim(NET3)[[1]]){
	for(p in 1:dim(NET3)[[2]]){
		if(NET3[j,p]>0){
			tt<-NET3[max(1,j-6):min(NR,j+6),max(1,p-6):min(NF,p+6)]+NET3[j,p]
			if(length(which(tt==0))==0){NET3[j,p]<-0}
		}
}	}

NET3[,(dim(NET3)[[2]]-2):dim(NET3)[[2]]]<-0

rm(NET2)

for(p in 1:dim(NET1)[[1]]){
	tt<-which(NET1[p,]>0);if(length(tt)>0){NET1[p,tt]<-0}
	tt<-which(NET3[p,]>=0);if(length(tt)>0){NET1[p,tt]<-0}
}

NET<-NET*(1+sign(NET1))+NET1

NETM<-as.numeric(NET)
NETM<-cbind(NETM,NETM,NETM)
NETM[,1]<-rep(seq(1,length(SR)),length(SF))
NETM[,2]<-rep(seq(1,length(SF)),each=length(SR))
NETM<-NETM[-which(NETM[,3]==0),]
NETM<-cbind(NETM,NETM[,3])
NETM<-NETM[order(NETM[,3],decreasing=F),]

###############  Deleting subclones closer than 3 cells to definite cluster

p<-1
while((p<=dim(NETM)[[1]])&&(NETM[p,3]<0)){p<-p+1}

if(p<dim(NETM)[[1]]){NETM[1:(p-1),4]<-4}

for(k in 1:(p-1)){
	if(p<=dim(NETM)[[1]]){
		NETM[p:dim(NETM)[[1]],4]<-abs(NETM[p:dim(NETM)[[1]],1]-NETM[k,1])
		tt<-which(NETM[p:dim(NETM)[[1]],4]<4)
		if(length(tt)>0){
	
			NETM[(p-1+tt),4]<-abs(NETM[(p-1+tt),2]-NETM[k,2])
			tt<-which(NETM[p:dim(NETM)[[1]],4]<4)
			if(length(tt)>0){NETM<-NETM[-(p-1+tt),]}
		}
	}
}#k	



NETM[,4]<-NETM[,3]

####################################
###############  Enumerating subclones 

NETM[,3]<-NETM[,4]

NETM<-NETM[order(NETM[,3],decreasing=T),]

p<-1
while(NETM[p,3]>0){p<-p+1}

if(p>1){

	Dist<-as.matrix(dist(NETM[1:(p-1),1:2],method="maximum",diag=T,upper=T))
	Dist[which(Dist>1)]<-0
	for(k in 1:dim(Dist)[[1]]){Dist[k,k]<-1}


	cn_CL<-(-40)

	for(k in 1:dim(Dist)[[1]]){
		if(NETM[k,3]>0){
			NETM[which(Dist[,k]!=0),3]<-cn_CL

			if((k+1)<=dim(Dist)[[1]]){
				for(m in (k+1):dim(Dist)[[1]]){
					if(sum(Dist[,k]*Dist[,m])>0){
						Dist[,k]<-Dist[,k]+Dist[,m]
						NETM[which(Dist[,k]!=0),3]<-cn_CL
					}
				}#m
			}
			cn_CL<-cn_CL-1}
		}
	}

	NET<-NET*0
	for(k in 1:dim(NETM)[[1]]){
	NET[NETM[k,1],NETM[k,2]]<-NETM[k,3]
}# if p>1

NET<-abs(NET)

NETM<-abs(NETM)
NETM[,1]<-SR[NETM[,1]]
NETM[,2]<-SF[NETM[,2]]
NETM<-NETM[order(NETM[,3],decreasing=F),]

colnames(NETM)<-c("LRR_point","AD_point","Index","subClone_power")

out<-NETM

}# function markNETtemplate



#########################################################################
##################
################## Function GetGAPtemplateZones
##################
#########################################################################


GetGAPtemplateZones<-function(NET,Centr_CL) {


NF<-dim(NET)[[2]]
NR<-dim(NET)[[1]]
SF<-as.numeric(colnames(NET))
SR<-as.numeric(rownames(NET))
step_SF<-SF[2]-SF[1]
step_SR<-SR[2]-SR[1]

##### Representing GAP by template description on NET: matrix NET_tmp

NET_tmp<-GAPonNET_template(Centr_CL,SR,SF)

##### Comming back to NET, adding rows or/and columns if necessary 

### out of range top

p<-1
while((p<=dim(NET_tmp)[[1]])&&(NET_tmp[p,1]+NET_tmp[p,4]>NR)){p<-p+1}

F_max<-max(NET_tmp[,2]+NET_tmp[,6])

if((F_max>NF)&&((p==1)||(p>dim(NET_tmp)[[1]]))){p<-2}

if((p>1)&&(p<=dim(NET_tmp)[[1]])){
	
	R_max<-SR[NR]+max(0,round((NET_tmp[1,1]+NET_tmp[1,4]-NR+1)/2+1,0)*2*step_SR)
	F_max<-max(R_max+1,F_max*step_SF)
	R_max<-max(R_max,F_max-1)

	tt<-matrix(0,round((R_max-SR[NR])/step_SR,0),NF)
	rownames(tt)<-seq(SR[NR]+step_SR,R_max,step_SR)
	NET<-rbind(NET,tt)
	
	SR<-as.numeric(rownames(NET))
	NR<-length(SR)

	tt<-matrix(0,NR,round((F_max-SF[NF])/step_SF,0))
	colnames(tt)<-seq(SF[NF]+step_SF,F_max,step_SF)
	NET<-cbind(NET,tt)

	SF<-as.numeric(colnames(NET))
	NF<-length(SF)

}

### out of range bottom

p<-dim(NET_tmp)[[1]]
if((NET_tmp[p,1]-NET_tmp[p,3])<0){

	NET_tmp[p,3]<-max(1,NET_tmp[p,3])	
	tt<-matrix(0,max(2,-(NET_tmp[p,1]-NET_tmp[p,3])+1),NF)
	rownames(tt)<-seq(SR[1]-step_SR*dim(tt)[[1]],SR[1]-step_SR,step_SR)
	NET<-rbind(tt,NET)
	
	NET_tmp[,1]<-NET_tmp[,1]+dim(tt)[[1]]

	SR<-as.numeric(rownames(NET))
	NR<-length(SR)

}

out<-markNETtemplate(NET,NET_tmp)

}#function GetGAPtemplateZones



#########################################################################
##################
################## Function subClonalClusters
##################
#########################################################################


subClonalClusters<-function(NETM,Centr_CL){

CN_CL<-rbind(Centr_CL[1,],Centr_CL[5,])
CN_CL<-table(CN_CL[1,],CN_CL[2,])
CN_CL<-rbind(as.numeric(rownames(CN_CL)),as.numeric(colnames(CN_CL)))

BAF_CL<-rbind(Centr_CL[1,],Centr_CL[2,],Centr_CL[4,])
BAF_CL<-BAF_CL[,order(BAF_CL[3,],decreasing=F)]

SB_CL<-numeric(0)

if(length(which(NETM[,3]>=40))>1){
if(length(which(NETM[,3]<40))>0){
	SB_CL<-NETM[-which(NETM[,3]<40),]
	SB_CL<-SB_CL[order(SB_CL[,3],decreasing=F),]

	SB_CL[,1]<-SB_CL[,1]*SB_CL[,4]
	SB_CL[,2]<-SB_CL[,2]*SB_CL[,4]

	tt<-SB_CL[1,]

	for(k in 2:dim(SB_CL)[[1]]){
		if(SB_CL[k,3]==tt[3]){
			SB_CL[k,1]<-SB_CL[k,1]+SB_CL[k-1,1]
			SB_CL[k,2]<-SB_CL[k,2]+SB_CL[k-1,2]
			SB_CL[k,4]<-SB_CL[k,4]+SB_CL[k-1,4]
			SB_CL[k-1,2]<-(-100)
		}else{
			SB_CL[k-1,1]<-SB_CL[k-1,1]/SB_CL[k-1,4]
			SB_CL[k-1,2]<-SB_CL[k-1,2]/SB_CL[k-1,4]

			tt<-SB_CL[k,]
			}
	}#k

	SB_CL[k,1]<-SB_CL[k,1]/SB_CL[k,4]
	SB_CL[k,2]<-SB_CL[k,2]/SB_CL[k,4]


	if(length(which(SB_CL[,2]<0))>1){
		if(length(which(SB_CL[,2]>=0))==1){
			SB_CL<-rbind(SB_CL,cbind(SB_CL[,1:2],SB_CL[,3:4]+1))
		}
		SB_CL<-SB_CL[-which(SB_CL[,2]<0),]
		SB_CL<-SB_CL[order(SB_CL[,3],decreasing=F),]
	}

	SB_CL<-cbind(SB_CL[,3],SB_CL[,2:4],SB_CL[,1:2])
	SB_CL[,2:4]<-0
	SB_CL<-t(SB_CL)
	rownames(SB_CL)<-c("Index_Subclone","CN_LRR","B_All","CN_BAF","SB_LRR","SB_BAF")

k<-111
	for(k in 1:dim(SB_CL)[[2]]){

		p<-1
		while((p<=dim(CN_CL)[[2]])&&(SB_CL[5,k]>CN_CL[2,p])){p<-p+1}
		if(p>dim(CN_CL)[[2]]){
			SB_CL[2,k]<-CN_CL[1,p-1]
		}else{
			if(p==1){
				SB_CL[2,k]<-CN_CL[1,p]
			}else{
				w<-c((CN_CL[2,p]-SB_CL[5,k]),(SB_CL[5,k]-CN_CL[2,p-1]))
				w<-w/(CN_CL[2,p]-CN_CL[2,p-1])
				SB_CL[2,k]<-weighted.mean(c(CN_CL[1,p-1],CN_CL[1,p]),w)}
			}

		SB_CL[2,k]<-round(SB_CL[2,k],1)
		if((p>1)&&(SB_CL[2,k]==CN_CL[1,p-1])){SB_CL[2,k]<-SB_CL[2,k]+0.1}
		if((p<=dim(CN_CL)[[2]])&&(SB_CL[2,k]==CN_CL[1,p])){SB_CL[2,k]<-max(0,SB_CL[2,k]-0.1)}

		if(SB_CL[6,k]>=SB_CL[5,k]+0.9){
			SB_CL[3,k]<-SB_CL[2,k]
			SB_CL[4,k]<-SB_CL[2,k]
		}else{
			if((p>1)&&(length(which(BAF_CL[1,]<CN_CL[1,p-1]))>0)){
				tmp_BAF_CL<-BAF_CL[,-which(BAF_CL[1,]<CN_CL[1,p-1])]
			}else{
				tmp_BAF_CL<-BAF_CL
				}
			if(p<=dim(CN_CL)[[2]]){
				tt<-which(tmp_BAF_CL[1,]>CN_CL[1,p])
				if(length(tt)>0){
					if(length(tt)==(dim(tmp_BAF_CL)[[2]]-1)){tmp_BAF_CL<-cbind(tmp_BAF_CL,tmp_BAF_CL)}
					tmp_BAF_CL<-tmp_BAF_CL[,-which(tmp_BAF_CL[1,]>CN_CL[1,p])]
				}
			}
			
			tmp_BAF_CL<-cbind(tmp_BAF_CL,tmp_BAF_CL[,dim(tmp_BAF_CL)[[2]]])
			tmp_BAF_CL[3,dim(tmp_BAF_CL)[[2]]]<-1

			p<-1
			while((p<dim(tmp_BAF_CL)[[2]])&&(SB_CL[6,k]>tmp_BAF_CL[3,p])){p<-p+1}
			if(p==1){p<-p+1}

			w<-c((tmp_BAF_CL[3,p]-SB_CL[6,k]),(SB_CL[6,k]-tmp_BAF_CL[3,p-1]))

			if(tmp_BAF_CL[3,p]-tmp_BAF_CL[3,p-1]!=0){w<-w/(tmp_BAF_CL[3,p]-tmp_BAF_CL[3,p-1])}

			if(tmp_BAF_CL[1,p-1]!=0){
				SB_CL[3,k]<-weighted.mean(c(tmp_BAF_CL[2,p-1]/tmp_BAF_CL[1,p-1],tmp_BAF_CL[2,p]/tmp_BAF_CL[1,p]),w)
			}else{
				if(sum(w)!=0){SB_CL[3,k]<-tmp_BAF_CL[2,p]*w[2]/sum(w)}else{SB_CL[3,k]<-tmp_BAF_CL[2,p]/2}
				}
			SB_CL[3,k]<-round(SB_CL[3,k]*SB_CL[2,k],1)

			if(p!=dim(tmp_BAF_CL)[[2]]){
				if(sum(w)!=0){SB_CL[4,k]<-weighted.mean(c(tmp_BAF_CL[1,p-1],tmp_BAF_CL[1,p]),w)
				}else{SB_CL[4,k]<-(tmp_BAF_CL[1,p]+tmp_BAF_CL[1,p-1])/2}

				SB_CL[4,k]<-round(SB_CL[4,k],1)
				if((SB_CL[4,k]==tmp_BAF_CL[1,p-1])&&(SB_CL[4,k]!=tmp_BAF_CL[1,p])){SB_CL[4,k]<-SB_CL[4,k]+0.1}
				if((SB_CL[4,k]==tmp_BAF_CL[1,p])&&(SB_CL[4,k]!=tmp_BAF_CL[1,p-1])){SB_CL[4,k]<-max(0,SB_CL[4,k]-0.1)}
			}else{}

			} # else for below diagonal


		}#k

}}# if there are subclones

out<-SB_CL
}# function subClonalClusters





#########################################################################
##################
################## Function CopyNumber_and_Genotype
##################
#########################################################################

#power<-3/4
#showGAPTemplate<-T

CopyNumber_and_Genotype_pattern<-function (seg_BAF,seg_LRR,seg_len,showGAPTemplate,power){

##################################################################
####
#### Step 2.  GAP pattern recognition
####
##################################################################


#######################################################
####  2.1 Data preparation: determination of net, density, etc.
####
####  SR,SF vector of discretized range of LRR and BAF correspondignly
####  NR=length(SR),NF<-length(SF)
####  NET is matrix BAFxLRR with discretized density of GAP


R_min<-min(seg_LRR[which(seg_len>50)],na.rm=T)
R_max<-max(seg_LRR[which(seg_len>50)],na.rm=T)

F_min<-0
F_max<-max(seg_BAF,na.rm=T)

R_max<-max(R_max,F_max-1)

R_min<-round((R_min-0.16)*10,0)/10
R_min<-max(R_min,-1)
R_max<-round((R_max+0.16)*10,0)/10

F_max<-R_max+1
F_min<-0

step_SR<-0.005
step_SF<-0.01

NR<-round(((R_max-R_min)/step_SR),0)
SR<-round(seq((R_min+step_SR),R_max,step_SR),3)

NF<-round(((F_max-F_min)/step_SF),0)+1
SF<-round(seq(F_min,F_max,step_SF),3)


NET<-matrix(0,NR,NF)

rownames(NET)<-SR
colnames(NET)<-SF


#### Determining coordinates of regions on NET
k<-1
for(k in 1:length(seg_LRR)){

	if(seg_len[k]>=50){
		sr<-which.min(abs(SR-seg_LRR[k]))
		if(seg_LRR[k]>=SR[sr]){sr<-min(sr+1,length(SR))}

		sf<-which.min(abs(SF-seg_BAF[k]))
		if(seg_BAF[k]>=SF[sf]){sf<-min(sf+1,length(SF))}

		NET[sr,sf]<-NET[sr,sf]+seg_len[k]
}
}

#########################################
#### 2.1.1  Smoothing density on NET: simple averaging density of adjucent 9 cells
#### kk weight of the center cell, sometime its worth to enhance kk for clarifying the picture;
#### however, almost always it works nice with kk=1


NET<-smoothNET(NET,weightC=1)


################################################
#### 2.1.2 Clarifying the structure by filtering low density regions
#### The threshold for low density is 50 (set semi arbitrarilly 
#### to avoid intersection with cluster numbering: 1-40)
####

NET[which(NET<100)]<-0

##################################################################
####
#### 2.2  Determination of GAP pattern
####
###################################################################

#print("fitting GAP...")
FF_all<-fittingGAP(NET,power)


######## FF_all contains several top solutions for GAP
######## kk by default is the first one, however, sometime for comlicated pattern the right one is 2,3..

CN<-maxCN
Centr_CL<-Centr_CL_annot(CN)

kk<-as.numeric(rownames(table(FF_all[9,])))[1]


if(showGAPTemplate){

tt<-plotGAPdensity(NET)
k<-1
for(k in 1:dim(FF_all)[[2]]){
	
if(FF_all[9,k]==kk){
		
		Delta<-which(FF_all[5:7,k]!=0)
		Delta<-c(Delta[1]+1, SR[FF_all[Delta[1]+4,k]])
		p_BAF<-FF_all[2,k]
		q_LRR<-FF_all[3,k]

		Centr_CL<-getCenterClusterGAP(CN,p_BAF,q_LRR,Delta,power)

		for(j in 1:dim(Centr_CL)[[2]]){
			text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""),cex=1)
		}
		abline(-1,1)
	}#if right Set_pattern
}#k

}# if show GAP template

################### Final decision
### p_BAF and q_LRR are set to the mean of final pattern set shown above

tt<-which(FF_all[9,]!=kk)
FF_final<-FF_all

if(length(tt)>0){
	if(length(tt)==(dim(FF_all)[[2]]-1)){FF_final<-cbind(FF_final,FF_final)}
	FF_final<-FF_final[,-which(FF_final[9,]!=kk)]
}else{
FF_final<-cbind(FF_final,FF_final)
}

p_BAF<-mean(FF_final[2,])
q_LRR<-mean(FF_final[3,])

for(k in 4:8){
	if(sum(FF_final[k,])>0){FF_final[k,1]<-round(mean(FF_final[k,which(FF_final[k,]!=0)]),0)}
}#k

FF_final[4:8,1]<-round(FF_final[4:8,1],0)

Delta<-which(FF_final[5:7,1]!=0)
Delta<-c(Delta[1]+1, SR[FF_final[Delta[1]+4,1]])
		
Centr_CL<-getCenterClusterGAP(CN,p_BAF,q_LRR,Delta,power)

#####################


if(showGAPTemplate){

tt<-plotGAPdensity(NET)

for(j in 1:dim(Centr_CL)[[2]]){
	text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""),cex=1)
}
abline(-1,1)
abline(-1+p_BAF,1)

text(1.2,-0.5,paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=0.9)
text(1.2,-0.6,paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=0.9)
text(1.2,-0.7,paste("2 copy LRR = ",as.character(Centr_CL[5,4])),cex=0.9)


}# if show GAP template



##############################################################
############ Marking cluster zones

NETM<-GetGAPtemplateZones(NET,Centr_CL)

############################################"
####################### Summarizing cluster zones and subclones


###### 0.0 Summarizing centers of clusters and subclones

SB_CL<-subClonalClusters(NETM,Centr_CL)

out<-list(NETM=NETM, Centr_CL=Centr_CL, SB_CL=SB_CL, coeff=c(p_BAF,q_LRR))

#out

}# function CopyNumber_and_Genotype





#########################################################################
##################
################## Function CopyNumber_and_Genotype WO fitting
##################
#########################################################################


CopyNumber_and_Genotype_patternWOfitting<-function (seg_BAF,seg_LRR,seg_len,showGAPTemplate,power,pp,qq,Delta){


CN<-maxCN


##################################################################
####
#### Step 2.  GAP pattern recognition
####
##################################################################


#######################################################
####  2.1 Data preparation: determination of net, density, etc.
####
####  SR,SF vector of discretized range of LRR and BAF correspondignly
####  NR=length(SR),NF<-length(SF)
####  NET is matrix BAFxLRR with discretized density of GAP


R_min<-min(seg_LRR[which(seg_len>50)],na.rm=T)
R_max<-max(seg_LRR[which(seg_len>50)],na.rm=T)

F_min<-0
F_max<-max(seg_BAF,na.rm=T)

R_max<-max(R_max,F_max-1)

R_min<-round((R_min-0.16)*10,0)/10
R_min<-max(R_min,-1)
R_max<-round((R_max+0.16)*10,0)/10

F_max<-R_max+1
F_min<-0

step_SR<-0.005
step_SF<-0.01

NR<-round(((R_max-R_min)/step_SR),0)
SR<-seq((R_min+step_SR),R_max,step_SR)

NF<-round(((F_max-F_min)/step_SF),0)+1
SF<-seq(F_min,F_max,step_SF)


NET<-matrix(0,NR,NF)

rownames(NET)<-SR
colnames(NET)<-SF


#### Determining coordinates of regions on NET
k<-1
for(k in 1:length(seg_LRR)){

	if(seg_len[k]>=50){
		sr<-which.min(abs(SR-seg_LRR[k]))
		if(seg_LRR[k]>=SR[sr]){sr<-min(sr+1,length(SR))}

		sf<-which.min(abs(SF-seg_BAF[k]))
		if(seg_BAF[k]>=SF[sf]){sf<-min(sf+1,length(SF))}

		NET[sr,sf]<-NET[sr,sf]+seg_len[k]
}
}

#########################################
#### 2.1.1  Smoothing density on NET: simple averaging density of adjucent 9 cells
#### kk weight of the center cell, sometime its worth to enhance kk for clarifying the picture;
#### however, almost always it works nice with kk=1


NET<-smoothNET(NET,weightC=1)


################################################
#### 2.1.2 Clarifying the structure by filtering low density regions
#### The threshold for low density is 50 (set semi arbitrarilly 
#### to avoid intersection with cluster numbering: 1-40)
####

NET[which(NET<100)]<-0

##################################################################
####
#### 2.2  Determination of GAP pattern
####
###################################################################

Centr_CL<-Centr_CL_annot(CN)

p_BAF<-pp
q_LRR<-qq
	
Centr_CL<-getCenterClusterGAP(CN,p_BAF,q_LRR,Delta,power)

#####################


if(showGAPTemplate){

tt<-plotGAPdensity(NET)

for(j in 1:dim(Centr_CL)[[2]]){
	text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""),cex=1)
}
abline(-1,1)
abline(-1+p_BAF,1)
}# if show GAP template


##############################################################
############ Marking cluster zones

NETM<-GetGAPtemplateZones(NET,Centr_CL)

####################################
####### Show Cluster attribution 

if(showGAPTemplate){

RB<-rainbow(40)

#for(p in 1:38){
#	tt1<-which(NETM[,3]==p)
#	if(length(tt1)>0){
#		rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])
#	}
#}

for(j in 1:dim(Centr_CL)[[2]]){
	text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
}


text(1,-0.5,paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=0.8)
text(1,-0.6,paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=0.8)
text(1,-0.7,paste("2copyLRR = ",as.character(round(Centr_CL[5,4],2)),sep=""),cex=0.8)


}# if show GAP template



############################################"
####################### Summarizing cluster zones and subclones


###### 0.0 Summarizing centers of clusters and subclones

SB_CL<-subClonalClusters(NETM,Centr_CL)

out<-list(NETM=NETM, Centr_CL=Centr_CL, SB_CL=SB_CL, coeff=c(p_BAF,q_LRR))

#out

}# function CopyNumber_and_Genotype WO Fitting


#########################################################################
##################
################## Function get ranking of possible Copy Numbers based on CN value
##################
##################
#######################################################

#cnV<-(-1.5)

getCNrank<-function(cnV,Centr_CL){

	Centr_CL[4,]<-Centr_CL[2,]-cnV
	kk<-which.min(abs(Centr_CL[4,]))

	Centr_CL[5,kk]<-10
	if(kk==1){if((Centr_CL[4,kk]<0)&&(Centr_CL[3,kk+1]<abs(Centr_CL[4,kk]))){Centr_CL[5,kk+1]<-9}else{Centr_CL[5,kk+1]<-8}
	}else{if(kk==9){if((Centr_CL[4,kk]>0)&&(Centr_CL[3,kk]<Centr_CL[4,kk])){Centr_CL[5,kk-1]<-9}else{Centr_CL[5,kk-1]<-8}

		}else{if(Centr_CL[4,kk]>=0){if(Centr_CL[3,kk]<Centr_CL[4,kk]){Centr_CL[5,kk-1]<-9;Centr_CL[5,kk+1]<-8
						}else{Centr_CL[5,kk-1]<-8;Centr_CL[5,kk+1]<-8}
			}else{if(Centr_CL[3,kk+1]<abs(Centr_CL[4,kk])){Centr_CL[5,kk+1]<-9;Centr_CL[5,kk-1]<-8
				}else{Centr_CL[5,kk+1]<-8;Centr_CL[5,kk-1]<-8}

				}
			}
		}
	kk<-which(Centr_CL[5,]==1)
	Centr_CL[5,kk]<-order(abs(Centr_CL[4,kk]),decreasing=T)+1

	out<-Centr_CL[5,]
	out
}



#########################################################################
##################
################## Function getCNwithPrecision
##################
#########################################################################


getCNwithPrecision<-function(tmp,Centr_CL,Precision){

c_CL<-table(Centr_CL[5,])
c_CL<-rbind(seq(0,8),as.numeric(rownames(c_CL)))
c_CL<-rbind(c_CL,c_CL,seq(0,8))
c_CL[3,2:dim(c_CL)[[2]]]<-c_CL[2,2:dim(c_CL)[[2]]]-c_CL[2,1:(dim(c_CL)[[2]]-1)]
c_CL[3,1]<-1
c_CL[3,]<-round(c_CL[3,]*Precision,4)
c_CL[5,]<-1	

Centr_CL<-cbind(Centr_CL,Centr_CL[,which(Centr_CL[1,]-Centr_CL[2,]==0)])
Centr_CL[2,26:34]<-0
Centr_CL[3,26:34]<-Centr_CL[1,26:34]+30
Centr_CL[4,26:34]<-Centr_CL[5,26:34]+1
Centr_CL<-Centr_CL[,order(Centr_CL[3,],decreasing=F)]


for(k in 1:dim(tmp)[[1]]){

	c_CL[5,]<-1
	c_CL[5,]<-getCNrank(tmp[k,c_cn],c_CL)
	tt<-which(c_CL[5,]==10)
	tt1<-which(c_CL[5,]==9)
	if(length(tt1)==1){
		kk<-round(abs(c_CL[4,tt]-tmp[k,c_cn])/abs(c_CL[4,tt]-c_CL[4,tt1]),1)
		tmp[k,c_nC]<-c_CL[1,tt]*(1-kk)+c_CL[1,tt1]*kk
	}else{tmp[k,c_nC]<-c_CL[1,tt]}

}#


out<-tmp

}#function




#########################################################################
##################
################## Function getGTforCNwithPresicion
##################
#########################################################################


getGTforCNwithPresicion<-function(tmp,Centr_CL,NETM){


Centr_CL<-cbind(Centr_CL,Centr_CL[,which(Centr_CL[1,]-Centr_CL[2,]==0)])
Centr_CL[2,26:34]<-0
Centr_CL[3,26:34]<-Centr_CL[1,26:34]+30
Centr_CL[4,26:34]<-Centr_CL[5,26:34]+1
Centr_CL<-Centr_CL[,order(Centr_CL[3,],decreasing=F)]

tmp[,c_nS]<-0

k<-1
for(k in 1:dim(tmp)[[1]]){

	if(trunc(tmp[k,c_nC])==tmp[k,c_nC]){kk<-c(tmp[k,c_nC])
	}else{kk<-c(trunc(tmp[k,c_nC]),trunc(tmp[k,c_nC])+1)}

	tt<-which(kk<0);if(length(tt)>0){kk<-kk[-tt]}
	tt<-which(kk>8);if(length(tt)>0){kk<-kk[-tt]}
	if(length(kk)==1){kk<-c(kk,kk)}

	kk<-Centr_CL[3,which(Centr_CL[1,]%in% kk)]
	if(length(kk)==1){kk<-c(kk,kk)}

	tt<-which(NETM[,3] %in% kk)
	if(length(tt)>0){tt<-NETM[tt,]}
	kk<-which(NETM[,3]>=40)
	if(length(kk)>0){tt<-rbind(tt,NETM[kk,])}

	tt[,4]<-sqrt((tt[,1]-tmp[k,c_cn])^2+1/4*(tt[,2]-tmp[k,c_adL])^2)

	tt<-tt[order(tt[,3],tt[,4],decreasing=F),]
	hh<-table(tt[,3])
	hh<-c(0,hh)

	for(p in 2:length(hh)){
		tt[(hh[p-1]+1),4]<-sum(tt[(hh[p-1]+1):min((hh[p-1]+10),(hh[p-1]+hh[p])),4])/min(10,hh[p])
		if(1<hh[p]){tt[(hh[p-1]+2):(hh[p-1]+hh[p]),3]<-0}
			hh[p]<-hh[p-1]+hh[p]
		}

	if(length(which(tt[,3]==0))>0){tt<-tt[-which(tt[,3]==0),]}

	tt<-tt[order(tt[,4],decreasing=F),]
	tt[,4]<-signif(tt[,4],digits=1)

	if((tt[1,3]>=40)&&(tmp[k,c_len]>=50)){tmp[k,c_nS]<-tt[1,3]}
	pp<-which(tt[,3]>=40)
	if(length(pp)>0){
		if(length(pp)<(dim(tt)[[1]]-2)){tt<-tt[-which(tt[,3]>=40),]
		}else{tt<-tt[-which(tt[,3]>=40),];tt<-rbind(tt,tt,tt)}
	}

	for(p in 1:3){
		if(tt[p,3]>=30){
			tmp[k,c_CN1+3*(p-1)]<-tt[p,3]-30
			tmp[k,c_BA1+3*(p-1)]<-0
		}else{
			tmp[k,c_CN1+3*(p-1)]<-Centr_CL[1,tt[p,3]]
			tmp[k,c_BA1+3*(p-1)]<-Centr_CL[2,tt[p,3]]
		}
		tmp[k,c_rnk1+3*(p-1)]<-tt[p,4]
	}#p


}#k


out<-tmp
}# function getCNandGTwithRank








########################################################################
############
############  Output summary plots
############
########################################################################


#######################################################################
###########
########### Chromosome pictures
###########
#######################################################################

summaryPlots_perChrom<-function(tmp,nameSample,datA, datC, Centr_CL){


Plot_dir<-paste(outputPath,"Chr_Plots_",nameSample,sep="")
dir.create(Plot_dir, showWarnings =F, recursive = FALSE, mode = "0777")

kk<-1

for(kk in 1:23){

jpeg(file=paste(Plot_dir,"/Plot_",nameSample,"_Chr_",
   as.character(kk),".jpg",sep=""),height=1000,width=1200,pointsize=24)

MAIN<-paste(nameSample,"Chr_",as.character(kk))

KN<-which(CN_annot[,C_Chr]==kk)
KK<-KN[length(KN)]
KN<-KN[1]

plot(c(1,CN_annot[KK,C_Pos]/1000),c(0,0),ylim=c(-15,15),main=MAIN,type="n",xlab="kb",ylab="",yaxt="n")
points(CN_annot[KN:KK,C_Pos]/1000,datC[KN:KK]*4-9,col="black",pch=".")

KN<-which(SNP_annot[,C_Chr]==kk)
KK<-KN[length(KN)]
KN<-KN[1]

points(SNP_annot[KN:KK,C_Pos]/1000,abs(datA[KN:KK])*1.5*sample(c(-1,1),(KK-KN+1),replace=T)+9,col="black",pch=".")

abline(h=(-2),col="black")
abline(h=0,col="black")
text(-4000,0,"4")
text(-4000,-2,"2")


# Recognized profile tmp

KN<-which(trunc(tmp[,c_chr])==kk)
KK<-KN[length(KN)]
KN<-KN[1]

p<-KN
for(p in KN:KK){
lines(c(CN_annot[tmp[p,c_isCN],C_Pos]/1000,CN_annot[tmp[p,c_ifCN],C_Pos]/1000),c(tmp[p,c_nS]*4-9,tmp[p,c_nS]*4-9),col="red",lwd=3)
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(tmp[p,c_adH],tmp[p,c_adH])*1.5+9,col="red",lwd=3)
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(tmp[p,c_adH],tmp[p,c_adH])*(-1.5)+9,col="red",lwd=3)
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(tmp[p,c_CN],tmp[p,c_CN])-4,col="blue",lwd=3)
}


# Connecting profile
if(KK>(KN+1)){
for(p in (KN+1):KK){
lines(c(SNP_annot[tmp[p-1,c_if],C_Pos]/1000,SNP_annot[tmp[p,c_is],C_Pos]/1000),c(tmp[p-1,c_CN],tmp[p,c_CN])-4,col="blue",lwd=1)
}
}

# Acquired homozygosity regions
for(p in KN:KK){
if((tmp[p,c_CN]==tmp[p,c_BA])||(tmp[p,c_BA]==0)){
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(tmp[p,c_CN],tmp[p,c_CN])-4,col="red",lwd=3)
}
}


for(p in KN:KK){

bb<-""

	if((tmp[p,c_BA]==0)||(tmp[p,c_BA]==tmp[p,c_CN])){
		if(tmp[p,c_CN]>1){
			for(s in 1:tmp[p,c_CN]){bb<-paste(bb,"A",sep="")}
		}else{if(tmp[p,c_CN]==1){bb<-"A"}else{bb<-"0"}}
	}else{
		if(tmp[p,c_BA]>1){
			for(s in 1:tmp[p,c_BA]){bb<-paste(bb,"A",sep="")}
		}else{if(tmp[p,c_BA]==1){bb<-"A"}}

		if(tmp[p,c_CN]>tmp[p,c_BA]+1){
			for(s in (tmp[p,c_BA]+1):tmp[p,c_CN]){bb<-paste(bb,"B",sep="")}
		}else{if(tmp[p,c_CN]==tmp[p,c_BA]+1){bb<-paste(bb,"B",sep="")}}
	}


if((tmp[p,c_if]-tmp[p,c_is])/2>100){

if(tmp[p,c_is]==1){text((SNP_annot[tmp[p,c_is],C_Pos]+SNP_annot[tmp[p,c_if],C_Pos])/2000,tmp[p,c_CN]-4.5,bb,cex=0.5)
}else{
text((SNP_annot[tmp[p,c_is],C_Pos]+SNP_annot[tmp[p,c_if],C_Pos])/2000,tmp[p,c_CN]-3.3,bb,cex=0.5)
}
}

}#p

dev.off()

}#k


}# function



#######################################################################
###########
########### Summary plot
###########
#######################################################################

summaryPlot<-function(tmp,nameSample,datA, datC, Centr_CL,NETM,q_LRR){

############# Centromere summary
tt<-sort(union(which(tmp[,c_len]>15),which(trunc(tmp[,c_chr])==24)))
if(length(tt)>0){tmp1<-tmp[tt,]}


Chr_counts<-0
Centr_counts<-0
centr<-tmp1[1,]

for(k in 1:(dim(tmp1)[[1]]-1)){
	if((tmp1[k+1,c_chr]>tmp1[k,c_chr])&&(round((tmp1[k+1,c_chr]-0.2),0)==tmp1[k,c_chr])){
		centr<-rbind(centr,tmp1[k:(k+1),])
		Chr_counts<-Chr_counts+(tmp1[k,c_CN]+tmp1[k+1,c_CN])/2
		Centr_counts<-Centr_counts+tmp1[k,c_CN]+tmp1[k+1,c_CN]
	}else{
		tt<-round((tmp1[k+1,c_chr]-0.2),0)
		if(tmp1[k+1,c_chr]>tmp1[k,c_chr]){
			if((tt==13)||(tt==14)||(tt==15)||(tt==21)||(tt==22)){
				centr<-rbind(centr,tmp1[k+1,])
				Chr_counts<-Chr_counts+tmp1[k+1,c_CN]
				Centr_counts<-Centr_counts+tmp1[k+1,c_CN]

			}
		}
	}
}#k

centr<-centr[-1,]
centr<-cbind(centr[,1:7],centr[,16:17])



tt<-table(SNP_annot[,C_Chr],SNP_annot[,C_Chr+1])
CHR<-c(0)
tCHR<-c(0)

for(k in 1:dim(tt)[[1]]){
CHR<-c(CHR,CHR[2*k-1]+tt[k,1])
CHR<-c(CHR,CHR[2*k]+tt[k,2])
tCHR<-c(tCHR,as.numeric(rownames(tt))[k])
tCHR<-c(tCHR,as.numeric(rownames(tt))[k]+0.5)
}

names(CHR)<-tCHR
rm(tCHR)

k<-2
while(k<=(length(CHR)-1)){
if((CHR[k]-CHR[k-1]<3)||(CHR[k+1]-CHR[k]<3)){
CHR[k]<-CHR[k+1]
names(CHR)[k+1]<-0
}
k<-k+2
}

if(length(which(names(CHR)=="0"))){
CHR<-CHR[-which(names(CHR)=="0")]}

CHR<-c(0,CHR)
names(CHR)[1]<-"0"



jpeg(file=paste(outputPath,"Summary_plot_",nameSample,".jpg",sep=""),height=2000,width=1600,pointsize=24)

split.screen(c(2,1)) 
screen(1)

par(pty="m")
plot(c(-6000,905000),c(-3,3),type="n",main=nameSample,xlab="Probe Index along Genome",ylab="",lab=c(7,1,7))

####### Plotting genome

#RB<-rainbow(length(CHR), s = 1, v = 1, start = 0, end = max(1,length(CHR) - 1)/length(CHR),gamma = 1, alpha = 1)

for(k in 2:(length(CHR)-1)){

rect(CHR[k],1.3,CHR[k]-1,2.6, col="black",border="black")
rect(CHR[k],-2.2,CHR[k]-1,-0.8, col="black",border="black")

if(round((as.numeric(names(CHR))[k]-0.2),0)%%2==0){
rect(CHR[k-1]+1,1.3,CHR[k],2.8, col="grey90",border="grey90")
rect(CHR[k-1]+1,-2.2,CHR[k],-0.8, col="grey90",border="grey90")
if(round((as.numeric(names(CHR))[k])-0.2,0)==as.numeric(names(CHR))[k-1]){

rect(CHR[k-1],1.3,CHR[k-1]-1,2.8, col="black",border="black")
rect(CHR[k-1],-2.2,CHR[k-1]-1,-0.8, col="black",border="black")


}
}

if(round((as.numeric(names(CHR))[k]),0)==as.numeric(names(CHR))[k]){

if(as.numeric(names(CHR))[k]%%2==0){
text((CHR[k-1]+CHR[k])/2,-2.5,names(CHR)[k],col="black",cex=0.7)
}else{
text((CHR[k-1]+CHR[k])/2,3,names(CHR)[k],col="black",cex=0.7)
}

}
#rect(CHR[k-1],-2.8,CHR[k],-3, col=RB[k],border=RB[k])

}

RB<-rainbow(40)

for(p in 1:dim(tmp)[[1]]){
	cl<-which(abs(Centr_CL[1,]-tmp[p,c_CN])==0)
	ba<-which(abs(Centr_CL[2,cl]-tmp[p,c_BA])==0)
	if(length(ba)>0){tmp[p,c_sb]<-Centr_CL[3,cl[ba[1]]]}else{tmp[p,c_sb]<-30+tmp[p,c_CN]}
}

tt<-which((CN_annot[,C_Pos+1]+1)%%5==0)
points(CN_annot[tt,C_Pos+1],datC[tt]/2-1.5,col="black",pch=".")

tt<-which(SNP_annot[,C_Ind]%%10==0)
points(SNP_annot[tt,C_Ind],datA[tt]*sample(c(-1,1),(length(datA[tt])),replace=T)/3+2,col="black",pch=".")

for(p in 1:dim(tmp)[[1]]){
rect(tmp[p,c_is],-2.8,tmp[p,c_if],-3, col=RB[tmp[p,c_sb]],border=RB[tmp[p,c_sb]])
lines(c(tmp[p,c_is],tmp[p,c_if]),c(tmp[p,c_nS],tmp[p,c_nS])/2-1.5,col="red",type="l",lwd=2)
lines(c(tmp[p,c_is],tmp[p,c_if]),c(tmp[p,c_adH],tmp[p,c_adH])/3+2,col="red",lwd=2)
lines(c(tmp[p,c_is],tmp[p,c_if]),c(tmp[p,c_adH],tmp[p,c_adH])/(-3)+2,col="red",lwd=2)
}


abline(h=(-1),col="grey60",lty=2)
abline(h=(-1.5),col="white")
abline(h=(-2),col="grey60",lty=2)

abline(h=0,col="blue",lty=1)
abline(h=-0.2,col="blue",lty=2)
abline(h=-0.4,col="blue",lty=2)
abline(h=0.2,col="blue",lty=2)
abline(h=0.4,col="blue",lty=1)
abline(h=0.6,col="blue",lty=2)
abline(h=0.8,col="blue",lty=2)
abline(h=1,col="blue",lty=2)

rect(-40000,-3,-4000,3, col="white",border="white")


text(-10000,1.5,"0",col="black")
text(-10000,2.5,"1",col="black")
text(-10000,3,"AD",col="black")
text(-10000,-1.5,"0",col="black")
text(-10000,-1,"1",col="black")
text(-10000,-2,"-1",col="black")
text(-10000,-0.6,"LR",col="black")
text(-10000,0,"2",col="black")
text(-10000,0.4,"4",col="black")
text(-10000,1.1,"Copy",col="black")



# Copy number
tt<-which(tmp[,c_len]<100);if(length(tt)>0){tmp1<-tmp[-tt,]}

for(p in 1:dim(tmp1)[[1]]){
lines(c(tmp1[p,c_is],tmp1[p,c_if]),c(tmp1[p,c_CN],tmp1[p,c_CN])/5-0.4,col="navy",type="l",lwd=3)
}

for(p in 2:dim(tmp1)[[1]]){
lines(c(tmp1[p-1,c_if],tmp1[p,c_is]),c(tmp1[p-1,c_CN],tmp1[p,c_CN])/5-0.4,col="navy",lwd=1)
}


# Acquired homozygosity regions
for(p in 2:dim(tmp1)[[1]]){

if((tmp1[p,c_CN]==tmp1[p,c_BA])||(tmp1[p,c_BA]==0)){
lines(c(tmp1[p,c_is],tmp1[p,c_if]),c(tmp1[p,c_CN],tmp1[p,c_CN])/5-0.4,col="red",lwd=3)
}
}


######## GAP figure

screen(2)
split.screen(c(1,2))
screen(3)

parGAP<-c(0.45,0.6,0.8,1)
parPoint<-c(3,3,3.5,4)
if(q_LRR<=0.15){gg<-1}else{if(q_LRR<0.2){gg<-2}else{if(q_LRR<0.3){gg<-3}else{gg<-4}}}

plot(tmp[,c_adH],tmp[,c_nS],main="GAP",xlim=c(-1.5,1.5),ylim=c(-parGAP[gg],parGAP[gg]),type="n",xlab="Allelic difference",ylab="Log ratio")

CEX<-tmp[,c_len]/quantile(tmp[,c_len],prob=0.7)

tt<-which(CEX>parPoint[gg])
if(length(tt)>0){CEX[tt]<-parPoint[gg]}

tt<-which(CEX<0.03)
if(length(tt)>0){CEX[tt]<-0}

points(-tmp[,c_adH],tmp[,c_nS],cex=CEX,col="black")
points(tmp[,c_adH],tmp[,c_nS],cex=CEX,col="grey")

abline(h=0)
abline(v=0)
abline(-1,1)
abline(-1+p_BAF,1)



############################
######### GAP pattern

RB<-rainbow(40)

for(p in 1:25){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])}
}

for(p in 31:38){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])}
}

for(j in 1:dim(Centr_CL)[[2]]){
	if(Centr_CL[1,j]<7){
	if(Centr_CL[1,j]==Centr_CL[2,j]*2){
		text(0.05,Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
	}else{
		text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
		}
}}

text(-1,-parGAP[gg]+0.2*parGAP[gg],paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=0.8)
text(-1,-parGAP[gg]+0.1*parGAP[gg],paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=0.8)
text(-1,-parGAP[gg],paste("2copyLRR = ",as.character(round(Centr_CL[5,4],3)),sep=""),cex=0.8)

################ Centromere table and summary inf
################## Summary inf

DNAi<-0
tt<-0
k<-1
for(k in 1:dim(tmp)[[1]]){
DNAi<-DNAi+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000*tmp[k,c_CN]
tt<-tt+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000
}
DNAi<-round(DNAi/tt/2,2)


HomoP<-0
for(k in 1:dim(tmp)[[1]]){
  if((tmp[k,c_CN]==tmp[k,c_BA])||(tmp[k,c_BA]==0)){
	HomoP<-HomoP+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000
  }
}
HomoP<-round(HomoP/tt*100,0)

BaseP<-0
BaseLossP<-0

if(Chr_counts>60){Ploidy<-2}else{Ploidy<-1}

for(k in 1:dim(tmp)[[1]]){
  if((tmp[k,c_CN]==2*Ploidy)&&(tmp[k,c_BA]==1*Ploidy)){
	BaseP<-BaseP+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000
  }
  if((tmp[k,c_CN]==1*Ploidy)&&(tmp[k,c_BA]==1*Ploidy)){
	BaseLossP<-BaseLossP+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000
  }
}
BaseP<-round(BaseP/tt*100,0)
BaseLossP<-round(BaseLossP/tt*100,0)

tt<-which(tmp[,c_len]<30);if(length(tt)>0){tmp1<-tmp[-tt,]}
tmp1<-shrinkReprTMP(tmp1)

screen(4)

plot(c(1,56),c(1,65),type="n",main="Summary and Centromeres",cex.main=0.9,xlab="",ylab="",col.axis="white",xaxt="n",yaxt="n")

text(3,65,paste("Break counts      = ",as.character(dim(tmp1)[[1]]-41)),cex=0.8,pos=4)
text(3,62,paste("Chromosome counts = ",as.character(Chr_counts)),cex=0.8,pos=4)
text(3,59,paste("Centromere counts = ",as.character(Centr_counts)),cex=0.8,pos=4)
text(3,56,paste("DNA index         = ",as.character(DNAi)),cex=0.8,pos=4)


text(30,65,paste("Ploidy detected   = ",as.character(Ploidy*2)),cex=0.8,pos=4)
text(30,62,paste("% base state      = ",as.character(BaseP)),cex=0.8,pos=4)
text(30,59,paste("% '1' copy loss   = ",as.character(BaseLossP)),cex=0.8,pos=4)
text(30,56,paste("% accuired homo   = ",as.character(HomoP)),cex=0.8,pos=4)


rect(0,0,57,50,col="white",border="black")

for(p in 1:2){
lines(c(7+(p-1)*30,7+(p-1)*30),c(1,48),col="black")
lines(c(14+(p-1)*30,14+(p-1)*30),c(1,48),col="black")
lines(c(21+(p-1)*30,21+(p-1)*30),c(1,48),col="black")
lines(c(28+(1-1)*30,28+(1-1)*30),c(1,48),col="black")


text(3+(p-1)*30,48,"Chr",cex=0.8)
text(10+(p-1)*30,48,"Copy",cex=0.8)
text(17+(p-1)*30,48," B allele",cex=0.8)
text(24+(p-1)*30,48,"SNPs",cex=0.8)
}

Centro_breaks<-0

for(k in 1:dim(centr)[[1]]){

#k<-23
#k<-1
if(k<23){p<-1}else{p<-2}

if(trunc(centr[k,c_chr])==centr[k,c_chr]){
text(3+(p-1)*30,46*p-2*k-(p-1)*2,paste(as.character(trunc(centr[k,c_chr])),"p"),cex=0.7)
if((k<dim(centr)[[1]])&&(trunc(centr[k+1,c_chr])==centr[k,c_chr])&&(centr[k,8]==centr[k+1,8])&&
    ((centr[k,9]==centr[k+1,9])||(centr[k,9]*centr[k+1,9]==0))){
Centro_breaks<-Centro_breaks+1
}
}else{
text(3+(p-1)*30,46*p-2*k-(p-1)*2,paste(as.character(trunc(centr[k,c_chr])),"q"),cex=0.7)

}

text(10+(p-1)*30,46*p-2*k-(p-1)*2,as.character(centr[k,8]),cex=0.7)
text(17+(p-1)*30,46*p-2*k-(p-1)*2,as.character(centr[k,9]),cex=0.7)
text(24+(p-1)*30,46*p-2*k-(p-1)*2,as.character(centr[k,c_len]),cex=0.7)

}

Centro_breaks<-(18-Centro_breaks)
text(3,53,paste("Centromere breaks = ",as.character(Centro_breaks)),cex=0.8,pos=4)

close.screen(all = TRUE)
dev.off()


summary_info<-c(p_BAF,q_LRR,Centr_CL[5,4],(dim(tmp1)[[1]]-41),DNAi,Ploidy*2,Chr_counts)

out<-summary_info

}## summary plot function



########################################################################
############
############  Output summary plotGAP
############
########################################################################


GAPplot<-function(tmp,nameSample,Centr_CL,NETM,q_LRR){

############# Centromere summary

tt<-sort(union(which(tmp[,c_len]>15),which(trunc(tmp[,c_chr])==24)))
if(length(tt)>0){tmp1<-tmp[tt,]}



Chr_counts<-0
Centr_counts<-0
centr<-tmp1[1,]

for(k in 1:(dim(tmp1)[[1]]-1)){
	if((tmp1[k+1,c_chr]>tmp1[k,c_chr])&&(round((tmp1[k+1,c_chr]-0.2),0)==tmp1[k,c_chr])){
		centr<-rbind(centr,tmp1[k:(k+1),])
		Chr_counts<-Chr_counts+(tmp1[k,c_CN]+tmp1[k+1,c_CN])/2
		Centr_counts<-Centr_counts+tmp1[k,c_CN]+tmp1[k+1,c_CN]
	}else{
		tt<-round((tmp1[k+1,c_chr]-0.2),0)
		if(tmp1[k+1,c_chr]>tmp1[k,c_chr]){
			if((tt==13)||(tt==14)||(tt==15)||(tt==21)||(tt==22)){
				centr<-rbind(centr,tmp1[k+1,])
				Chr_counts<-Chr_counts+tmp1[k+1,c_CN]
				Centr_counts<-Centr_counts+tmp1[k+1,c_CN]

			}
		}
	}
}#k

centr<-centr[-1,]
centr<-cbind(centr[,1:7],centr[,16:17])

DNAi<-0
tt<-0
k<-1
for(k in 1:dim(tmp)[[1]]){
DNAi<-DNAi+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000*tmp[k,c_CN]
tt<-tt+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000
}
DNAi<-round(DNAi/tt/2,2)

#######################################################################
###########
########### Summary plot
###########
#######################################################################


jpeg(file=paste(outputPath,"GAP_plot_",nameSample,".jpg",sep=""),height=1200,width=1000,pointsize=24)


parGAP<-c(0.45,0.6,0.8,1)
parPoint<-c(3,3,3.5,4)
if(q_LRR<=0.15){gg<-1}else{if(q_LRR<0.2){gg<-2}else{if(q_LRR<0.3){gg<-3}else{gg<-4}}}

plot(tmp[,c_adH],tmp[,c_nS],main="GAP",xlim=c(-1.5,1.5),ylim=c(-parGAP[gg],parGAP[gg]),type="n",xlab="Allelic difference",ylab="Log ratio")

CEX<-tmp[,c_len]/quantile(tmp[,c_len],prob=0.7)

tt<-which(CEX>parPoint[gg])
if(length(tt)>0){CEX[tt]<-parPoint[gg]}

tt<-which(CEX<0.03)
if(length(tt)>0){CEX[tt]<-0}


points(-tmp[,c_adH],tmp[,c_nS],cex=CEX,col="black")
points(tmp[,c_adH],tmp[,c_nS],cex=CEX,col="grey")

abline(h=0)
abline(v=0)
abline(-1,1)
abline(-1+p_BAF,1)

############################
######### GAP pattern

RB<-rainbow(40)

for(p in 1:25){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){
		rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])
	}
}

for(p in 31:38){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){
		rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])
	}
}

for(j in 1:dim(Centr_CL)[[2]]){

	if(Centr_CL[1,j]<7){
	if(Centr_CL[1,j]==Centr_CL[2,j]*2){
		text(0.05,Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
	}else{
		text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
		}
}
}

text(-1.2,-parGAP[gg]+0.1+0.2*parGAP[gg],paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=0.9)
text(-1.2,-parGAP[gg]+0.1+0.1*parGAP[gg],paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=0.9)
text(-1.2,-parGAP[gg]+0.1,paste("DNAind = ",as.character(DNAi)),cex=0.9)

text(1,-parGAP[gg]+0.1+0.2*parGAP[gg],paste("Breaks = ",as.character(dim(tmp)[[1]]-41)),cex=0.9)
text(1,-parGAP[gg]+0.1+0.1*parGAP[gg],paste("Chrs = ",as.character(Chr_counts)),cex=0.9)
text(1,-parGAP[gg]+0.1,paste("Centrs = ",as.character(Centr_counts)),cex=0.9)





dev.off()


}## GAP plot function


